@@ -1,5 +1,5 @@
 from ._colors import get_srgb1
-from ._main import plot, plot_colors, plot_contour_abs, plot_contour_arg
+from ._main import plot, plot_abs, plot_arg, plot_contours, plot_phase
 from ._tri import tricontour_abs, tripcolor
 from .benchmark import show_kovesi_test_image, show_test_function
 from .create import create_colormap, find_max_srgb_radius, show_circular, show_linear
@@ -12,10 +12,11 @@ __all__ = [
     "find_max_srgb_radius",
     "create_colormap",
     "get_srgb1",
-    "plot_colors",
-    "plot_contour_abs",
-    "plot_contour_arg",
     "plot",
+    "plot_arg",
+    "plot_abs",
+    "plot_phase",
+    "plot_contours",
     #
     "tripcolor",
     "tricontour_abs",
